@@ -21,9 +21,9 @@
 #include "dynarec_arm64_functions.h"
 
 
-uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int rep, int* ok, int* need_epilog)
+uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int* ok, int* need_epilog)
 {
-    (void)ip; (void)rep; (void)need_epilog;
+    (void)ip; (void)need_epilog;
 
     uint8_t opcode = F8;
     uint8_t nextop;
@@ -39,28 +39,21 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
     MAYUSE(wb2);
     MAYUSE(j64);
 
-    while((opcode==0xF2) || (opcode==0xF3)) {
-        rep = opcode-0xF1;
-        opcode = F8;
-    }
-
-    GETREX();
-
     switch(opcode) {
 
         case 0x01:
-            INST_NAME("LOCK ADD Ew, Gw");
-            SETFLAGS(X_ALL, SF_SET_PENDING);
             nextop = F8;
-            GETGW(x5);
             if(MODREG) {
-                ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                UXTHw(x6, ed);
-                emit_add16(dyn, ninst, x6, x5, x3, x4);
-                BFIx(ed, x6, 0, 16);
+                INST_NAME("Invalid LOCK");
+                UDF(0);
+                *need_epilog = 1;
+                *ok = 0;
             } else {
+                INST_NAME("LOCK ADD Ew, Gw");
+                SETFLAGS(X_ALL, SF_SET_PENDING);
+                GETGW(x5);
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                if(arm64_atomics) {
+                if(cpuext.atomics) {
                     UFLAG_IF {
                         LDADDALH(x5, x1, wback);
                         emit_add16(dyn, ninst, x1, x5, x3, x4);
@@ -74,23 +67,22 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                     STLXRH(x3, x1, wback);
                     CBNZx_MARKLOCK(x3);
                 }
-                SMDMB();
             }
             break;
 
         case 0x09:
-            INST_NAME("LOCK OR Ew, Gw");
-            SETFLAGS(X_ALL, SF_SET_PENDING);
             nextop = F8;
-            GETGW(x5);
             if(MODREG) {
-                ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                UXTHw(x6, ed);
-                emit_or16(dyn, ninst, x6, x5, x3, x4);
-                BFIx(ed, x6, 0, 16);
+                INST_NAME("Invalid LOCK");
+                UDF(0);
+                *need_epilog = 1;
+                *ok = 0;
             } else {
+                INST_NAME("LOCK OR Ew, Gw");
+                SETFLAGS(X_ALL, SF_SET_PENDING);
+                GETGW(x5);
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                if(arm64_atomics) {
+                if(cpuext.atomics) {
                     UFLAG_IF {
                         LDSETALH(x5, x1, wback);
                         emit_or16(dyn, ninst, x1, x5, x3, x4);
@@ -104,7 +96,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                     STLXRH(x3, x1, wback);
                     CBNZx_MARKLOCK(x3);
                 }
-                SMDMB();
             }
             break;
 
@@ -113,22 +104,20 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
             switch(opcode) {
 
                 case 0xB1:
-                    INST_NAME("LOCK CMPXCHG Ew, Gw");
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     nextop = F8;
-                    GETGD;
-                    UXTHw(x6, xRAX);
                     if(MODREG) {
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        wback = 0;
-                        UXTHw(x1, ed);
-                        CMPSxw_REG(x6, x1);
-                        B_MARK(cNE);
-                        BFIx(ed, gd, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        INST_NAME("LOCK CMPXCHG Ew, Gw");
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
+                        GETGD;
+                        UXTHw(x6, xRAX);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
                         if(!ALIGNED_ATOMICH) {
-                            if(arm64_uscat) {
+                            if(cpuext.uscat) {
                                 ANDx_mask(x1, wback, 1, 0, 3);  // mask = F
                                 CMPSw_U12(x1, 0xF);
                                 B_MARK3(cGE);
@@ -138,7 +127,7 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             }
                         }
                         // Aligned version
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             MOVw_REG(x1, x6);
                             CASALH(x1, gd, wback);
                         } else {
@@ -166,28 +155,27 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             CBNZx_MARK3(x4);
                             STRH_U12(gd, wback, 0);
                         }
+                        MARK;
+                        // Common part (and fallback for EAX != Ed)
+                        UFLAG_IF {emit_cmp16(dyn, ninst, x6, x1, x3, x4, x5);}
+                        BFIx(xRAX, x1, 0, 16);
                     }
-                    MARK;
-                    SMDMB();
-                    // Common part (and fallback for EAX != Ed)
-                    UFLAG_IF {emit_cmp16(dyn, ninst, x6, x1, x3, x4, x5);}
-                    BFIx(xRAX, x1, 0, 16);
                     break;
 
                 case 0xC1:
-                    INST_NAME("LOCK XADD Gw, Ew");
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     nextop = F8;
-                    gd = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3));
-                    UXTHx(x5, gd);
                     if(MODREG) {
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        BFIx(gd, ed, 0, 16);
-                        emit_add16(dyn, ninst, x5, gd, x3, x4);
-                        BFIx(ed, x5, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        INST_NAME("LOCK XADD Gw, Ew");
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
+                        gd = TO_NAT(((nextop & 0x38) >> 3) + (rex.r << 3));
+                        UXTHx(x5, gd);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             LDADDALH(x5, x1, wback);
                         } else {
                             MARKLOCK;
@@ -196,7 +184,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRH(x3, x4, wback);
                             CBNZx_MARKLOCK(x3);
                         }
-                        SMDMB();
                         IFX(X_ALL|X_PEND) {
                             MOVxw_REG(x2, x1);
                             emit_add16(dyn, ninst, x2, x5, x3, x4);
@@ -211,40 +198,39 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
             break;
 
         case 0x11:
-            INST_NAME("LOCK ADC Ew, Gw");
-            READFLAGS(X_CF);
-            SETFLAGS(X_ALL, SF_SET_PENDING);
             nextop = F8;
-            GETGW(x5);
             if(MODREG) {
-                ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                UXTHw(x6, ed);
-                emit_adc16(dyn, ninst, x6, x5, x3, x4);
-                BFIx(ed, x6, 0, 16);
+                INST_NAME("Invalid LOCK");
+                UDF(0);
+                *need_epilog = 1;
+                *ok = 0;
             } else {
+                INST_NAME("LOCK ADC Ew, Gw");
+                READFLAGS(X_CF);
+                SETFLAGS(X_ALL, SF_SET_PENDING);
+                GETGW(x5);
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
                 MARKLOCK;
                 LDAXRH(x1, wback);
                 emit_adc16(dyn, ninst, x1, x5, x3, x4);
                 STLXRH(x3, x1, wback);
                 CBNZx_MARKLOCK(x3);
-                SMDMB();
             }
             break;
 
         case 0x21:
-            INST_NAME("LOCK AND Ew, Gw");
-            SETFLAGS(X_ALL, SF_SET_PENDING);
             nextop = F8;
-            GETGW(x5);
             if(MODREG) {
-                ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                UXTHw(x6, ed);
-                emit_and16(dyn, ninst, x6, gd, x3, x4);
-                BFIx(ed, x6, 0, 16);
+                INST_NAME("Invalid LOCK");
+                UDF(0);
+                *need_epilog = 1;
+                *ok = 0;
             } else {
+                INST_NAME("LOCK AND Ew, Gw");
+                SETFLAGS(X_ALL, SF_SET_PENDING);
+                GETGW(x5);
                 addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                if(arm64_atomics) {
+                if(cpuext.atomics) {
                     MVNw_REG(x3, gd);
                     UFLAG_IF {
                         LDCLRALH(x3, x1, wback);
@@ -259,7 +245,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                     STLXRH(x3, x1, wback);
                     CBNZx_MARKLOCK(x3);
                 }
-                SMDMB();
             }
             break;
 
@@ -268,25 +253,23 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
             nextop = F8;
             switch((nextop>>3)&7) {
                 case 0: //ADD
-                    if(opcode==0x81) {
-                        INST_NAME("LOCK ADD Ew, Iw");
-                    } else {
-                        INST_NAME("LOCK ADD Ew, Ib");
-                    }
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        MOV32w(x5, i16);
-                        UXTHw(x6, ed);
-                        emit_add16(dyn, ninst, x6, x5, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {
+                            INST_NAME("LOCK ADD Ew, Iw");
+                        } else {
+                            INST_NAME("LOCK ADD Ew, Ib");
+                        }
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         MOV32w(x5, i16);
                         if(!ALIGNED_ATOMICH) {
-                            if(arm64_uscat) {
+                            if(cpuext.uscat) {
                                 ANDx_mask(x1, wback, 1, 0, 3);  // mask = F
                                 CMPSw_U12(x1, 0xF);
                                 B_MARK(cGE);
@@ -295,7 +278,7 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                                 B_MARK(cNE);
                             }
                         }
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             UFLAG_IF {
                                 LDADDALH(x5, x1, wback);
                             } else {
@@ -308,7 +291,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRH(x3, x4, wback);
                             CBNZx_MARKLOCK(x3);
                         }
-                        SMDMB();
                         if(!ALIGNED_ATOMICH) {
                             B_MARK2_nocond;
                             MARK;   // unaligned! also, not enough
@@ -320,7 +302,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRB(x3, x4, wback);
                             CBNZx_MARK(x3);
                             STRH_U12(x4, wback, 0);    // put the whole value
-                            SMDMB();
                         }
                         MARK2;
                         UFLAG_IF {
@@ -329,19 +310,18 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                     }
                     break;
                 case 1: //OR
-                    if(opcode==0x81) {INST_NAME("LOCK OR Ew, Iw");} else {INST_NAME("LOCK OR Ew, Ib");}
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        UXTHw(x6, ed);
-                        emit_or16c(dyn, ninst, x6, i16, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK OR Ew, Iw");} else {INST_NAME("LOCK OR Ew, Ib");}
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         if(!i64) {MOV32w(x5, i16);}
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             UFLAG_IF {
                                 LDSETALH(x5, x1, wback);
                                 emit_or16c(dyn, ninst, x1, i16, x3, x4);
@@ -361,21 +341,18 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRH(x3, x1, wback);
                             CBNZx_MARKLOCK(x3);
                         }
-                        SMDMB();
                     }
                     break;
                 case 2: //ADC
-                    if(opcode==0x81) {INST_NAME("LOCK ADC Ew, Iw");} else {INST_NAME("LOCK ADC Ew, Ib");}
-                    READFLAGS(X_CF);
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        MOV32w(x5, i16);
-                        UXTHw(x6, ed);
-                        emit_adc16(dyn, ninst, x6, x5, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK ADC Ew, Iw");} else {INST_NAME("LOCK ADC Ew, Ib");}
+                        READFLAGS(X_CF);
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         MOV32w(x5, i16);
@@ -384,21 +361,18 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                         emit_adc16(dyn, ninst, x1, x5, x3, x4);
                         STLXRH(x3, x1, wback);
                         CBNZx_MARKLOCK(x3);
-                        SMDMB();
                     }
                     break;
                 case 3: //SBB
-                    if(opcode==0x81) {INST_NAME("LOCK SBB Ew, Iw");} else {INST_NAME("LOCK SBB Ew, Ib");}
-                    READFLAGS(X_CF);
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        MOV32w(x5, i16);
-                        UXTHw(x6, ed);
-                        emit_sbb16(dyn, ninst, x6, x5, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK SBB Ew, Iw");} else {INST_NAME("LOCK SBB Ew, Ib");}
+                        READFLAGS(X_CF);
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         MOV32w(x5, i16);
@@ -407,23 +381,21 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                         emit_sbb16(dyn, ninst, x1, x5, x3, x4);
                         STLXRH(x3, x1, wback);
                         CBNZx_MARKLOCK(x3);
-                        SMDMB();
                     }
                     break;
                 case 4: //AND
-                    if(opcode==0x81) {INST_NAME("LOCK AND Ew, Iw");} else {INST_NAME("LOCK AND Ew, Ib");}
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        UXTHw(x6, ed);
-                        emit_and16c(dyn, ninst, x6, i16, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK AND Ew, Iw");} else {INST_NAME("LOCK AND Ew, Ib");}
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         i64 = convert_bitmask_w(i16);
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             MOV32w(x5, ~i16);
                             UFLAG_IF {
                                 LDCLRALH(x5, x1, wback);
@@ -447,26 +419,23 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             }
                             STLXRH(x3, x1, wback);
                             CBNZx_MARKLOCK(x3);
-                            SMDMB();
                         }
                     }
                     break;
                 case 5: //SUB
-                    if(opcode==0x81) {INST_NAME("LOCK SUB Ew, Iw");} else {INST_NAME("LOCK SUB Ew, Ib");}
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        MOV32w(x5, i16);
-                        UXTHw(x6, ed);
-                        emit_sub16(dyn, ninst, x6, x5, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK SUB Ew, Iw");} else {INST_NAME("LOCK SUB Ew, Ib");}
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         MOV32w(x5, i16);
                         if(!ALIGNED_ATOMICH) {
-                            if(arm64_uscat) {
+                            if(cpuext.uscat) {
                                 ANDx_mask(x1, wback, 1, 0, 3);  // mask = F
                                 CMPSw_U12(x1, 0xF);
                                 B_MARK(cGE);
@@ -475,7 +444,7 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                                 B_MARK(cNE);
                             }
                         }
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             NEGw_REG(x4, x5);
                             UFLAG_IF {
                                 LDADDALH(x4, x1, wback);
@@ -489,7 +458,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRH(x3, x1, wback);
                             CBNZx_MARKLOCK(x3);
                         }
-                        SMDMB();
                         if(!ALIGNED_ATOMICH) {
                             B_MARK2_nocond;
                             MARK;   // unaligned! also, not enough
@@ -501,7 +469,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                             STLXRB(x3, x4, wback);
                             CBNZx_MARK(x3);
                             STRH_U12(x4, wback, 0);    // put the whole value
-                            SMDMB();
                         }
                         MARK2;
                         UFLAG_IF {
@@ -510,19 +477,18 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                     }
                     break;
                 case 6: //XOR
-                    if(opcode==0x81) {INST_NAME("LOCK XOR Ew, Iw");} else {INST_NAME("LOCK XOR Ew, Ib");}
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
                     if(MODREG) {
-                        if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                        UXTHw(x6, ed);
-                        emit_xor16c(dyn, ninst, x6, i16, x3, x4);
-                        BFIx(ed, x6, 0, 16);
+                        INST_NAME("Invalid LOCK");
+                        UDF(0);
+                        *need_epilog = 1;
+                        *ok = 0;
                     } else {
+                        if(opcode==0x81) {INST_NAME("LOCK XOR Ew, Iw");} else {INST_NAME("LOCK XOR Ew, Ib");}
+                        SETFLAGS(X_ALL, SF_SET_PENDING);
                         addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, (opcode==0x81)?2:1);
                         if(opcode==0x81) i16 = F16S; else i16 = F8S;
                         i64 = convert_bitmask_w(i16);
-                        if(arm64_atomics) {
+                        if(cpuext.atomics) {
                             UFLAG_IF {
                                 LDEORALH(x5, x1, wback);
                                 emit_xor16c(dyn, ninst, x1, i16, x3, x4);
@@ -539,23 +505,14 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                                 emit_xor16(dyn, ninst, x1, x5, x3, x4);
                             STLXRH(x3, x1, wback);
                             CBNZx_MARKLOCK(x3);
-                            SMDMB();
                         }
                     }
                     break;
                 case 7: //CMP
-                    if(opcode==0x81) {INST_NAME("(LOCK) CMP Ew, Iw");} else {INST_NAME("(LOCK) CMP Ew, Ib");}
-                    SETFLAGS(X_ALL, SF_SET_PENDING);
-                    GETEW(x6, (opcode==0x81)?2:1);
-                    (void)wb1;
-                    // No need to LOCK, this is readonly
-                    if(opcode==0x81) i16 = F16S; else i16 = F8S;
-                    if(i16) {
-                        MOV32w(x5, i16);
-                        emit_cmp16(dyn, ninst, x6, x5, x3, x4, x6);
-                    } else {
-                        emit_cmp16_0(dyn, ninst, ed, x3, x4);
-                    }
+                    INST_NAME("Invalid LOCK");
+                    UDF(0);
+                    *need_epilog = 1;
+                    *ok = 0;
                     break;
             }
             break;
@@ -565,16 +522,16 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                 switch((nextop>>3)&7)
                 {
                     case 0: // INC Ew
-                        INST_NAME("LOCK INC Ew");
-                        SETFLAGS(X_ALL&~X_CF, SF_SUBSET);
                         if(MODREG) {
-                            ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                            UXTHw(x6, ed);
-                            emit_inc16(dyn, ninst, x6, x5, x3);
-                            BFIx(ed, x6, 0, 16);
+                            INST_NAME("Invalid LOCK");
+                            UDF(0);
+                            *need_epilog = 1;
+                            *ok = 0;
                         } else {
+                            INST_NAME("LOCK INC Ew");
+                            SETFLAGS(X_ALL&~X_CF, SF_SUBSET);
                             addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                            if(arm64_atomics) {
+                            if(cpuext.atomics) {
                                 MOV32w(x3, 1);
                                 UFLAG_IF {
                                     LDADDALH(x3, x1, wback);
@@ -589,20 +546,19 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                                 STLXRH(x3, x1, wback);
                                 CBNZx_MARKLOCK(x3);
                             }
-                            SMDMB();
                         }
                         break;
                     case 1: //DEC Ew
-                        INST_NAME("LOCK DEC Ew");
-                        SETFLAGS(X_ALL&~X_CF, SF_SUBSET);
                         if(MODREG) {
-                            ed = TO_NAT((nextop & 7) + (rex.b << 3));
-                            UXTHw(x6, ed);
-                            emit_dec16(dyn, ninst, x6, x5, x3);
-                            BFIx(ed, x6, 0, 16);
+                            INST_NAME("Invalid LOCK");
+                            UDF(0);
+                            *need_epilog = 1;
+                            *ok = 0;
                         } else {
+                            INST_NAME("LOCK DEC Ew");
+                            SETFLAGS(X_ALL&~X_CF, SF_SUBSET);
                             addr = geted(dyn, addr, ninst, nextop, &wback, x2, &fixedaddress, NULL, 0, 0, rex, LOCK_LOCK, 0, 0);
-                            if(arm64_atomics) {
+                            if(cpuext.atomics) {
                                 MOV32w(x3, -1);
                                 UFLAG_IF {
                                     LDADDALH(x3, x1, wback);
@@ -617,7 +573,6 @@ uintptr_t dynarec64_66F0(dynarec_arm_t* dyn, uintptr_t addr, uintptr_t ip, int n
                                 STLXRH(x3, x1, wback);
                                 CBNZx_MARKLOCK(x3);
                             }
-                            SMDMB();
                         }
                         break;
                     default:
