@@ -34,6 +34,7 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
     int v0, v1;
     int q0, q1;
     int d0, d1;
+    uint8_t tmp1, tmp2, tmp3;
     int64_t fixedaddress, gdoffset;
     int unscaled;
     int64_t j64;
@@ -173,6 +174,95 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
                 }
             }
             break;
+        case 0x38:  /* MAP */
+            opcode = F8;
+            switch(opcode) {
+                case 0xF6:
+                    INST_NAME("ADOX Gd, Ed");
+                    nextop = F8;
+                    READFLAGS(X_OF);
+                    SETFLAGS(X_OF, SF_SUBSET, NAT_FLAGS_NOFUSION);
+                    GETED(0);
+                    GETGD;
+                    if(cpuext.lbt) {
+                        X64_GET_EFLAGS(x3, X_OF);
+                        SRLI_D(x3, x3, F_OF);
+                    } else {
+                        BSTRPICK_D(x3, xFlags, F_OF, F_OF);
+                    }
+                    IFX(X_OF) {
+                        if(rex.w) {
+                            ADD_D(x4, gd, ed);
+                            SLTU(x5, x4, gd);
+                            ADD_D(gd, x4, x3);
+                            SLTU(x6, gd, x4);
+                        } else {
+                            ADD_W(x4, gd, ed);
+                            ZEROUP(x4);
+                            ZEROUP(gd);
+                            SLTU(x5, x4, gd);
+                            ADD_W(gd, x4, x3);
+                            ZEROUP(gd);
+                            SLTU(x6, gd, x4);
+                        }
+                        OR(x5, x5, x6);
+                        if(cpuext.lbt) {
+                            SLLI_D(x5, x5, F_OF);
+                            X64_SET_EFLAGS(x5, X_OF);
+                        } else {
+                            BSTRINS_D(xFlags, x5, F_OF, F_OF);
+                        }
+                    } else {
+                        ADDxw(x5, gd, ed);
+                        ADDxw(gd, x5, x3);
+                        if (!rex.w) ZEROUP(gd);
+                    }
+                    break;
+                default:
+                    DEFAULT;
+            }
+            break;
+
+#define GO(GETFLAGS, NO, YES, NATNO, NATYES, F, I)                                               \
+    READFLAGS_FUSION(F, x1, x2, x3, x4, x5);                                                     \
+    if (!dyn->insts[ninst].nat_flags_fusion) {                                                   \
+        if (cpuext.lbt) {                                                                        \
+            X64_SETJ(tmp1, I);                                                                   \
+        } else {                                                                                 \
+            GETFLAGS;                                                                            \
+        }                                                                                        \
+    }                                                                                            \
+    nextop = F8;                                                                                 \
+    GETGD;                                                                                       \
+    if (MODREG) {                                                                                \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                                \
+        if (dyn->insts[ninst].nat_flags_fusion) {                                                \
+            NATIVEJUMP(NATNO, 8);                                                                \
+        } else {                                                                                 \
+            if (cpuext.lbt)                                                                      \
+                BEQZ(tmp1, 8);                                                                   \
+            else                                                                                 \
+                B##NO(tmp1, 8);                                                                  \
+        }                                                                                        \
+        MV(gd, ed);                                                                              \
+        if (!rex.w) ZEROUP(gd);                                                                  \
+    } else {                                                                                     \
+        addr = geted(dyn, addr, ninst, nextop, &ed, tmp2, tmp3, &fixedaddress, rex, NULL, 1, 0); \
+        if (dyn->insts[ninst].nat_flags_fusion) {                                                \
+            NATIVEJUMP(NATNO, 8);                                                                \
+        } else {                                                                                 \
+            if (cpuext.lbt)                                                                      \
+                BEQZ(tmp1, 8);                                                                   \
+            else                                                                                 \
+                B##NO(tmp1, 8);                                                                  \
+        }                                                                                        \
+        LDxw(gd, ed, fixedaddress);                                                              \
+    }
+
+            GOCOND(0x40, "CMOV", "Gd, Ed");
+
+#undef GO
+
         case 0x51:
             INST_NAME("SQRTSS Gx, Ex");
             nextop = F8;
@@ -188,11 +278,11 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             GETGX(v0, 1);
             GETEXSS(v1, 0, 0);
             q0 = fpu_get_scratch(dyn);
-            q1 = fpu_get_scratch(dyn);
-            LU12I_W(x3, 0x3f800); // 1.0f
-            MOVGR2FR_W(q0, x3);
-            FSQRT_S(q1, v1);
-            FDIV_S(q0, q0, q1);
+            if (cpuext.frecipe) {
+                FRSQRTE_S(q0, v1);
+            } else {
+                FRSQRT_S(q0, v1);
+            }
             VEXTRINS_W(v0, q0, 0);
             break;
         case 0x53:
@@ -201,9 +291,7 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             GETGX(v0, 1);
             GETEXSS(v1, 0, 0);
             d1 = fpu_get_scratch(dyn);
-            LU12I_W(x3, 0x3f800); // 1.0f
-            MOVGR2FR_W(d1, x3);
-            FDIV_S(d1, d1, v1);
+            FRECIP_S(d1, v1);
             VEXTRINS_W(v0, d1, 0);
             break;
         case 0x58:
@@ -238,20 +326,25 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             nextop = F8;
             GETEX(v1, 0, 0);
             GETGX_empty(v0);
-            VFTINTRZ_W_S(v0, v1);
-            if (!BOX64ENV(dynarec_fastround)) {
+            if (BOX64ENV(dynarec_fastround)) {
+                VFTINTRZ_W_S(v0, v1);
+            } else {
+                MOVGR2FCSR(FCSR2, xZR); // reset all bits
+                VFTINTRZ_W_S(v0, v1);
+                MOVFCSR2GR(x5, FCSR2); // get back FPSR to check
+                MOV32w(x3, (1 << FR_V) | (1 << FR_O));
+                AND(x5, x5, x3);
+                BEQZ_MARK(x5); // no fp exception, work done, fast path.
+
                 q0 = fpu_get_scratch(dyn);
-                q1 = fpu_get_scratch(dyn);
-                d1 = fpu_get_scratch(dyn);
-                VFCMP_S(q0, v1, v1, cEQ);
-                VLDI(q1, 0b1001110000000); // broadcast 0x80000000
-                VAND_V(v0, q0, v0);
-                VANDN_V(d1, q0, q1);
-                VOR_V(v0, v0, d1);
-                VSUBI_WU(d1, q1, 1);
-                VSEQ_W(q0, v0, d1);
-                VSRLI_W(q0, q0, 31);
-                VADD_W(v0, v0, q0);
+                q1 = fpu_get_scratch(dyn); // mask
+                d0 = fpu_get_scratch(dyn);
+                VLDI(q0, 0b1001110000000); // broadcast 0x80000000 to all
+                VLDI(d0, (0b10011 << 8) | 0x4f);
+                VFCMP_S(q1, d0, v1, cULE); // get Nan,+overflow mark
+                VBITSEL_V(v0, v0, q0, q1);
+
+                MARK;
             }
             break;
         case 0x5C:
@@ -268,15 +361,14 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             nextop = F8;
             GETGX(d0, 1);
             GETEXSS(d1, 0, 0);
-            FCMP_S(fcc0, d0, d1, cUN);
-            BCNEZ_MARK(fcc0);
-            FCMP_S(fcc1, d1, d0, cLT);
-            BCEQZ_MARK2(fcc1);
-            MARK;
-            v1 = fpu_get_scratch(dyn);
-            FMOV_S(v1, d1);
-            VEXTRINS_W(d0, v1, 0);
-            MARK2;
+            q0 = fpu_get_scratch(dyn);
+            if (BOX64ENV(dynarec_fastnan)) {
+                FMIN_S(q0, d0, d1);
+            } else {
+                FCMP_S(fcc0, d1, d0, cULE);
+                FSEL(q0, d0, d1, fcc0);
+            }
+            VEXTRINS_W(d0, q0, 0);
             break;
         case 0x5E:
             INST_NAME("DIVSS Gx, Ex");
@@ -292,15 +384,14 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             nextop = F8;
             GETGX(d0, 1);
             GETEXSS(d1, 0, 0);
-            FCMP_S(fcc0, d0, d1, cUN);
-            BCNEZ_MARK(fcc0);
-            FCMP_S(fcc1, d0, d1, cLT);
-            BCEQZ_MARK2(fcc1);
-            MARK;
-            v1 = fpu_get_scratch(dyn);
-            FMOV_S(v1, d1);
-            VEXTRINS_W(d0, v1, 0);
-            MARK2;
+            q0 = fpu_get_scratch(dyn);
+            if (BOX64ENV(dynarec_fastnan)) {
+                FMAX_S(q0, d0, d1);
+            } else {
+                FCMP_S(fcc0, d1, d0, cLT);
+                FSEL(q0, d1, d0, fcc0);
+            }
+            VEXTRINS_W(d0, q0, 0);
             break;
         case 0x6F:
             INST_NAME("MOVDQU Gx, Ex");
@@ -344,11 +435,7 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
             }
             GETGX_empty(v0);
             if (v0 == v1) {
-                // clear upper bits..
-                q1 = fpu_get_scratch(dyn);
-                VXOR_V(q1, q1, q1);
-                VEXTRINS_D(q1, v1, 0); // q1[63:0] = v1[63:0]
-                VOR_V(v0, q1, q1);
+                VINSGR2VR_D(v0, xZR, 1);
             } else {
                 VXOR_V(v0, v0, v0);
                 VEXTRINS_D(v0, v1, 0); // v0[63:0] = v1[63:0]
@@ -370,15 +457,32 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
         case 0xAE:
             nextop = F8;
             switch ((nextop >> 3) & 7) {
+                case 0:
+                case 1:
+                    if(rex.is32bits || !MODREG) {
+                        INST_NAME("Illegal AE");
+                        FAKEED;
+                        UDF();
+                    } else {
+                        if(((nextop>>3)&7)==1) {INST_NAME("RDGSBASE");} else {INST_NAME("RDFSBASE");}
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                        int seg = _FS + ((nextop>>3)&7);
+                        grab_segdata(dyn, addr, ninst, x4, seg, (MODREG));
+                        MV(ed, x4);
+                    }
+                     break;
                 case 2:
-                    INST_NAME("(unsupported) WRFSBASE Ed");
-                    FAKEED;
-                    UDF();
-                    break;
                 case 3:
-                    INST_NAME("(unsupported) WRGSBASE Ed");
-                    FAKEED;
-                    UDF();
+                    if(rex.is32bits || !MODREG) {
+                        INST_NAME("Illegal AE");
+                        FAKEED;
+                        UDF();
+                    } else {
+                        if(((nextop>>3)&7)==3) {INST_NAME("WRGSBASE");} else {INST_NAME("WRFSBASE");}
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                        int seg = _FS + ((nextop>>3)&7)-2;
+                        ST_D(ed, xEmu, offsetof(x64emu_t, segs_offs[seg]));
+                    }
                     break;
                 case 5:
                     INST_NAME("(unsupported) INCSSPD/INCSSPQ Ed");
@@ -421,7 +525,7 @@ uintptr_t dynarec64_F30F(dynarec_la64_t* dyn, uintptr_t addr, uintptr_t ip, int
                 VPICKVE2GR_WU(gd, v1, 0);
             }
             IFX (X_ALL) {
-                if (la64_lbt) {
+                if (cpuext.lbt) {
                     X64_SET_EFLAGS(xZR, X_ALL);
                     BNEZ_MARK(gd);
                     ADDI_D(x3, xZR, 1 << F_ZF);
