@@ -21,10 +21,9 @@
 #include "dynarec_rv64_functions.h"
 
 
-uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int rep, int* ok, int* need_epilog)
+uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ninst, rex_t rex, int* ok, int* need_epilog)
 {
     (void)ip;
-    (void)rep;
     (void)need_epilog;
 
     uint8_t nextop = F8;
@@ -133,7 +132,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                             ANDI(x5, x5, 7); // (emu->top + i)&7
                         }
                         // load x2 with ST0 anyway, for sign extraction
-                        if (rv64_zba)
+                        if (cpuext.zba)
                             SH3ADD(x1, x5, xEmu);
                         else {
                             SLLI(x5, x5, 3);
@@ -245,7 +244,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             MESSAGE(LOG_DUMP, "Need Optimization\n");
             x87_forget(dyn, ninst, x1, x2, 0);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_f2xm1, -1, 0, 0);
+            CALL(const_native_f2xm1, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
         case 0xF1:
@@ -254,7 +253,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_fyl2x, -1, 0, 0);
+            CALL(const_native_fyl2x, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             X87_POP_OR_FAIL(dyn, ninst, x3);
             break;
@@ -264,7 +263,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_ftan, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_ftan, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             X87_PUSH_OR_FAIL(v1, dyn, ninst, x1, EXT_CACHE_ST_F);
@@ -283,7 +282,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_fpatan, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_fpatan, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             X87_POP_OR_FAIL(dyn, ninst, x3);
@@ -294,7 +293,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             X87_PUSH_EMPTY_OR_FAIL(dyn, ninst, x3);
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_fxtract, -1, 0, 0);
+            CALL(const_native_fxtract, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
         case 0xF5:
@@ -303,7 +302,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_fprem1, -1, 0, 0);
+            CALL(const_native_fprem1, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
         case 0xF6:
@@ -328,7 +327,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_fprem, -1, 0, 0);
+            CALL(const_native_fprem, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
         case 0xF9:
@@ -337,7 +336,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
-            CALL(native_fyl2xp1, -1, 0, 0);
+            CALL(const_native_fyl2xp1, -1, 0, 0);
             x87_unstackcount(dyn, ninst, x3, s0);
             X87_POP_OR_FAIL(dyn, ninst, x3);
             break;
@@ -360,7 +359,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_fsincos, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_fsincos, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
@@ -411,7 +410,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 1);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_fscale, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_fscale, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
@@ -421,7 +420,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_fsin, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_fsin, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
@@ -431,7 +430,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
             x87_forget(dyn, ninst, x1, x2, 0);
             s0 = x87_stackcount(dyn, ninst, x3);
             if (!BOX64ENV(dynarec_fastround)) u8 = x87_setround(dyn, ninst, x1, x2);
-            CALL_(native_fcos, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
+            CALL_(const_native_fcos, -1, BOX64ENV(dynarec_fastround) ? 0 : u8, 0, 0);
             if (!BOX64ENV(dynarec_fastround)) x87_restoreround(dyn, ninst, u8);
             x87_unstackcount(dyn, ninst, x3, s0);
             break;
@@ -482,7 +481,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                 fpu_purgecache(dyn, ninst, 0, x1, x2, x3); // maybe only x87, not SSE?
                 addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 0, 0);
                 MOV32w(x2, 0);
-                CALL(fpu_loadenv, -1, ed, x2);
+                CALL(const_fpu_loadenv, -1, ed, x2);
                 NATIVE_RESTORE_X87PC();
                 break;
             case 5:
@@ -500,7 +499,7 @@ uintptr_t dynarec64_D9(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int ni
                 fpu_purgecache(dyn, ninst, 0, x1, x2, x3); // maybe only x87, not SSE?
                 addr = geted(dyn, addr, ninst, nextop, &ed, x1, x2, &fixedaddress, rex, NULL, 0, 0);
                 MOV32w(x2, 0);
-                CALL(fpu_savenv, -1, ed, x2);
+                CALL(const_fpu_savenv, -1, ed, x2);
                 break;
             case 7:
                 INST_NAME("FNSTCW Ew");
