@@ -35,13 +35,20 @@ void emit_shl8c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         SET_DFNONE();
     }
 
+    IFX (X_OF) {
+        SRLI(s3, s1, 6);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     if (c < 8) {
-        IFX (X_CF | X_OF) {
+        IFX (X_CF) {
             SRLI(s3, s1, 8 - c);
             ANDI(s5, s3, 1); // LSB == F_CF
-            IFX (X_CF) {
-                OR(xFlags, xFlags, s5);
-            }
+            OR(xFlags, xFlags, s5);
         }
 
         SLLI(s1, s1, c + 56);
@@ -56,15 +63,6 @@ void emit_shl8c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         IFX (X_ZF) {
             SET_FLAGS_EQZ(s1, F_ZF, s3);
         }
-        IFX (X_OF) {
-            // OF flag is affected only on 1-bit shifts
-            if (c == 1) {
-                SRLI(s3, s1, 7);
-                XOR(s3, s3, s5);
-                SLLI(s3, s3, F_OF2);
-                OR(xFlags, xFlags, s3);
-            }
-        }
         IFX (X_PF) {
             emit_pf(dyn, ninst, s1, s3, s4);
         }
@@ -81,7 +79,6 @@ void emit_shl8c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         IFX (X_PEND) {
             SB(s1, xEmu, offsetof(x64emu_t, res));
         }
-        // OF nop
         // SF nop
         // AF nop
         IFX (X_PF | X_ZF) {
@@ -121,19 +118,14 @@ void emit_shr8c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        // OF flag is set to the most-significant bit of the original operand
-        if (c == 1) {
-            SRLI(s3, s1, 7);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
+        SRLI(s3, s1, 7);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
 
     SRLI(s1, s1, c);
     ANDI(s1, s1, 0xff);
 
-
     // SF should be unset
     IFX (X_PEND) {
         SB(s1, xEmu, offsetof(x64emu_t, res));
@@ -206,14 +198,22 @@ void emit_shl8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
         SET_DFNONE();
     }
 
+
+    IFX (X_OF) {
+        SRLI(s3, s1, 6);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     SLL(s1, s1, s2);
 
-    IFX (X_CF | X_OF) {
+    IFX (X_CF) {
         SRLI(s5, s1, 8);
         ANDI(s5, s5, 1); // LSB == F_CF
-        IFX (X_CF) {
-            OR(xFlags, xFlags, s5);
-        }
+        OR(xFlags, xFlags, s5);
     }
 
     SLLI(s1, s1, 56);
@@ -228,15 +228,6 @@ void emit_shl8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s3);
     }
-    IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        ADDI(s3, s2, -1);
-        BNEZ(s3, 4 + 4 * 4);
-        SRLI(s3, s1, 7);
-        XOR(s3, s3, s5);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-    }
     IFX (X_PF) {
         emit_pf(dyn, ninst, s1, s3, s4);
     }
@@ -265,10 +256,6 @@ void emit_shr8(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4, i
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        // OF flag is set to the most-significant bit of the original operand
-        ADDI(s3, xZR, 1);
-        BNE(s2, s3, 4 + 3 * 4);
         SRLI(s3, s1, 7);
         SLLI(s3, s3, F_OF2);
         OR(xFlags, xFlags, s3);
@@ -348,13 +335,20 @@ void emit_shl16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         SET_DFNONE();
     }
 
+    IFX (X_OF) {
+        SRLI(s3, s1, 14);
+        SRLI(s5, s3, 1);
+        XOR(s3, s3, s5);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     if (c < 16) {
-        IFX (X_CF | X_OF) {
+        IFX (X_CF) {
             SRLI(s3, s1, 16 - c);
             ANDI(s5, s3, 1); // LSB == F_CF
-            IFX (X_CF) {
-                OR(xFlags, xFlags, s5);
-            }
+            OR(xFlags, xFlags, s5);
         }
 
         SLLI(s1, s1, c + 48);
@@ -369,25 +363,14 @@ void emit_shl16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         IFX (X_ZF) {
             SET_FLAGS_EQZ(s1, F_ZF, s3);
         }
-        IFX (X_OF) {
-            // OF flag is affected only on 1-bit shifts
-            if (c == 1) {
-                SRLI(s3, s1, 15);
-                XOR(s3, s3, s5);
-                SLLI(s3, s3, F_OF2);
-                OR(xFlags, xFlags, s3);
-            }
-        }
         IFX (X_PF) {
             emit_pf(dyn, ninst, s1, s3, s4);
         }
         if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
     } else {
-        IFX (X_CF) {
-            if (c == 16) {
-                ANDI(s3, s1, 1);
-                OR(xFlags, xFlags, s3); // F_CF == 0
-            }
+        IFXA (X_CF, (c == 16)) {
+            ANDI(s3, s1, 1);
+            OR(xFlags, xFlags, s3); // F_CF == 0
         }
         MV(s1, xZR);
 
@@ -434,13 +417,10 @@ void emit_shr16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
         // OF flag is set to the most-significant bit of the original operand
-        if (c == 1) {
-            SRLI(s3, s1, 15);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
+        SRLI(s3, s1, 15);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
 
     SRLI(s1, s1, c);
@@ -519,14 +499,21 @@ void emit_shl16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         SET_DFNONE();
     }
 
+    IFX (X_OF) {
+        SRLI(s3, s1, 14);
+        SRLI(s5, s3, 1);
+        XOR(s3, s3, s5);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     SLL(s1, s1, s2);
 
-    IFX (X_CF | X_OF) {
+    IFX (X_CF) {
         SRLI(s5, s1, 16);
         ANDI(s5, s5, 1); // LSB == F_CF
-        IFX (X_CF) {
-            OR(xFlags, xFlags, s5);
-        }
+        OR(xFlags, xFlags, s5);
     }
 
     SLLI(s1, s1, 48);
@@ -541,15 +528,6 @@ void emit_shl16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s3);
     }
-    IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        ADDI(s3, s2, -1);
-        BNEZ(s3, 4 + 4 * 4);
-        SRLI(s3, s1, 15);
-        XOR(s3, s3, s5);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-    }
     IFX (X_PF) {
         emit_pf(dyn, ninst, s1, s3, s4);
     }
@@ -578,10 +556,7 @@ void emit_shr16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4,
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
         // OF flag is set to the most-significant bit of the original operand
-        ADDI(s3, xZR, 1);
-        BNE(s2, s3, 4 + 3 * 4);
         SRLI(s3, s1, 15);
         SLLI(s3, s3, F_OF2);
         OR(xFlags, xFlags, s3);
@@ -658,7 +633,15 @@ void emit_shl32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    IFX (X_CF | X_OF) {
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 62 : 30);
+        SRLI(s5, s3, 1);
+        XOR(s3, s3, s5);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+    IFX (X_CF) {
         SUBI(s5, s2, rex.w ? 64 : 32);
         NEG(s5, s5);
         SRL(s3, s1, s5);
@@ -686,15 +669,6 @@ void emit_shl32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s3);
     }
-    IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        ADDI(s3, s2, -1);
-        BNEZ(s3, 4 + 4 * 4);
-        SRLIxw(s3, s1, rex.w ? 63 : 31);
-        XOR(s3, s3, s5);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-    }
     IFX (X_PF) {
         emit_pf(dyn, ninst, s1, s3, s4);
     }
@@ -718,12 +692,18 @@ void emit_shl32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
         SET_DFNONE();
     }
 
-    IFX (X_CF | X_OF) {
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 62 : 30);
+        SRLI(s5, s3, 1);
+        XOR(s3, s3, s5);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+    IFX (X_CF) {
         SRLI(s3, s1, (rex.w ? 64 : 32) - c);
         ANDI(s5, s3, 1); // LSB == F_CF
-        IFX (X_CF) {
-            OR(xFlags, xFlags, s5);
-        }
+        OR(xFlags, xFlags, s5);
     }
 
     if (rex.w) {
@@ -744,15 +724,6 @@ void emit_shl32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s3);
     }
-    IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
-        if (c == 1) {
-            SRLIxw(s3, s1, rex.w ? 63 : 31);
-            XOR(s3, s3, s5);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     IFX (X_PF) {
         emit_pf(dyn, ninst, s1, s3, s4);
     }
@@ -781,10 +752,7 @@ void emit_shr32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
         // OF flag is set to the most-significant bit of the original operand
-        ADDI(s3, xZR, 1);
-        BNE(s2, s3, 4 + 3 * 4);
         SRLIxw(s3, s1, rex.w ? 63 : 31);
         SLLI(s3, s3, F_OF2);
         OR(xFlags, xFlags, s3);
@@ -813,10 +781,7 @@ void emit_shr32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
 // emit SHR32 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
 void emit_shr32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, int s3, int s4)
 {
-    if (!c) {
-        if (!rex.w) ZEROUP(s1);
-        return;
-    }
+    if (!c) return;
 
     CLEAR_FLAGS();
 
@@ -842,13 +807,10 @@ void emit_shr32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // OF flag is affected only on 1-bit shifts
         // OF flag is set to the most-significant bit of the original operand
-        if (c == 1) {
-            SRLIxw(s3, s1, rex.w ? 63 : 31);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
+        SRLIxw(s3, s1, rex.w ? 63 : 31);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
 
     if (rex.w) {
@@ -950,7 +912,20 @@ void emit_rol32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (rv64_zbb) {
+    IFX (X_CF | X_OF) {
+        ANDI(xFlags, xFlags, ~((1UL << F_CF) | (1UL << F_OF2)));
+    }
+
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 62 : 30);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
+    if (cpuext.zbb) {
         if (rex.w) {
             ROL(s1, s1, s2);
         } else {
@@ -968,22 +943,9 @@ void emit_rol32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     IFX (X_PEND) {
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
-    IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~((1UL << F_CF) | (1UL << F_OF2)));
-    }
-    IFX (X_CF | X_OF) {
+    IFX (X_CF) {
         ANDI(s4, s1, 1); // LSB == F_CF
-        IFX (X_CF) OR(xFlags, xFlags, s4);
-    }
-    IFX (X_OF) {
-        // the OF flag is set to the exclusive OR of the CF bit (after the rotate) and the most-significant bit of the result.
-        ADDI(s3, s2, -1);
-        BNEZ_MARK(s3);
-        SRLIxw(s3, s1, rex.w ? 63 : 31);
-        XOR(s3, s3, s4); // s3: MSB, s4: CF bit
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-        MARK;
+        OR(xFlags, xFlags, s4);
     }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
@@ -1004,7 +966,19 @@ void emit_ror32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
         SET_DFNONE();
     }
 
-    if (rv64_zbb) {
+    IFX (X_CF | X_OF) {
+        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
+    }
+
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 63 : 31);
+        XOR(s3, s3, s1);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
+    if (cpuext.zbb) {
         if (rex.w) {
             ROR(s1, s1, s2);
         } else {
@@ -1022,25 +996,10 @@ void emit_ror32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int s
     IFX (X_PEND) {
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
-    IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
-    }
     IFX (X_CF) {
         SRLIxw(s3, s1, rex.w ? 63 : 31);
         OR(xFlags, xFlags, s3);
     }
-    IFX (X_OF) {
-        // the OF flag is set to the exclusive OR of the two most-significant bits of the result
-        ADDI(s3, s2, -1);
-        BNEZ_MARK(s3);
-        SRLIxw(s3, s1, rex.w ? 63 : 31);
-        SRLIxw(s4, s1, rex.w ? 62 : 30);
-        XOR(s3, s3, s4);
-        ANDI(s3, s3, 1);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-        MARK;
-    }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
 
@@ -1059,12 +1018,13 @@ void emit_rol16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
     }
 
     IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
+        if (c == 1)
+            ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
+        else
+            ANDI(xFlags, xFlags, ~(1UL << F_CF));
         ANDI(s4, s1, 1 << F_CF);
         IFX (X_CF) OR(xFlags, xFlags, s4);
-    }
-    IFX (X_OF) {
-        if (c == 1) {
+        IFXA (X_OF, (c == 1)) {
             SRLI(s3, s1, 15);
             XOR(s3, s3, s4);
             SLLI(s3, s3, F_OF2);
@@ -1086,6 +1046,14 @@ void emit_rol32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
         ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
     }
 
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 62 : 30);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
     IFX (X_PEND) {
         MOV32w(s3, c);
         SDxw(s3, xEmu, offsetof(x64emu_t, op2));
@@ -1093,9 +1061,9 @@ void emit_rol32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    if (rv64_zbb) {
+    if (cpuext.zbb) {
         RORIxw(s1, s1, (rex.w ? 64 : 32) - c);
-    } else if (rv64_xtheadbb) {
+    } else if (cpuext.xtheadbb) {
         TH_SRRIxw(s1, s1, (rex.w ? 64 : 32) - c);
     } else {
         SLLIxw(s3, s1, c);
@@ -1108,18 +1076,9 @@ void emit_rol32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
     IFX (X_PEND) {
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
     }
-    IFX (X_CF | X_OF) {
+    IFX (X_CF) {
         ANDI(s4, s1, 1 << F_CF);
-        IFX (X_CF) OR(xFlags, xFlags, s4);
-    }
-    IFX (X_OF) {
-        // the OF flag is set to the exclusive OR of the CF bit (after the rotate) and the most-significant bit of the result.
-        if (c == 1) {
-            SRLIxw(s3, s1, rex.w ? 63 : 31);
-            XOR(s3, s3, s4);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
+        OR(xFlags, xFlags, s4);
     }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
@@ -1131,6 +1090,20 @@ void emit_ror16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
 
     SET_DFNONE();
 
+    IFX (X_CF | X_OF) {
+        if (c == 1)
+            ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
+        else
+            ANDI(xFlags, xFlags, ~(1UL << F_CF));
+    }
+    IFXA (X_OF, (c == 1)) {
+        SRLI(s3, s1, 15);
+        XOR(s3, s3, s1);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     if (c & 15) {
         SRLI(s3, s1, c & 15);
         SLLI(s1, s1, 16 - (c & 15));
@@ -1138,40 +1111,31 @@ void emit_ror16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
         ZEXTH(s1, s1);
     }
 
-    IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
-    }
-
     IFX (X_CF) {
         SRLI(s3, s1, 15);
         OR(xFlags, xFlags, s3);
     }
 
-    IFX (X_OF) {
-        if (c == 1) {
-            SRLI(s3, s1, 14);
-            SRLI(s4, s3, 1);
-            XOR(s3, s3, s4);
-            ANDI(s3, s3, 1);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
 
 // emit ROR32 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
 void emit_ror32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c, int s3, int s4)
 {
-    if (!c) {
-        if (!rex.w) ZEROUP(s1);
-        return;
-    }
+    if (!c) return;
 
     IFX (X_CF | X_OF) {
         ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
     }
 
+    IFX (X_OF) {
+        SRLIxw(s3, s1, rex.w ? 63 : 31);
+        XOR(s3, s3, s1);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     IFX (X_PEND) {
         MOV32w(s3, c);
         SDxw(s3, xEmu, offsetof(x64emu_t, op2));
@@ -1179,9 +1143,9 @@ void emit_ror32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    if (rv64_zbb) {
+    if (cpuext.zbb) {
         RORIxw(s1, s1, c);
-    } else if (rv64_xtheadbb) {
+    } else if (cpuext.xtheadbb) {
         TH_SRRIxw(s1, s1, c);
     } else {
         SRLIxw(s3, s1, c);
@@ -1198,44 +1162,22 @@ void emit_ror32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, uint32_t c,
         SRLIxw(s3, s1, rex.w ? 63 : 31);
         OR(xFlags, xFlags, s3);
     }
-    IFX (X_OF) {
-        // the OF flag is set to the exclusive OR of the two most-significant bits of the result
-        if (c == 1) {
-            SRLI(s3, s1, rex.w ? 62 : 30);
-            SRLI(s4, s3, 1);
-            XOR(s3, s3, s4);
-            ANDI(s3, s3, 1);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
 
 // emit SHRD32 instruction, from s1, fill s2 , constant c, store result in s1 using s3 and s4 as scratch
 void emit_shrd32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uint32_t c, int s3, int s4, int s5)
 {
-    c &= (rex.w ? 0x3f : 0x1f);
     CLEAR_FLAGS();
 
     IFX (X_PEND) {
-        if (c) {
-            MOV64x(s3, c);
-            SDxw(s3, xEmu, offsetof(x64emu_t, op2));
-        } else
-            SDxw(xZR, xEmu, offsetof(x64emu_t, op2));
+        MOV64x(s3, c);
+        SDxw(s3, xEmu, offsetof(x64emu_t, op2));
         SDxw(s1, xEmu, offsetof(x64emu_t, op1));
         SET_DF(s4, rex.w ? d_shrd64 : d_shrd32);
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
-    if (!c) {
-        if (!rex.w) ZEROUP(s1);
-        IFX (X_PEND) {
-            SDxw(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        return;
-    }
     IFX (X_CF) {
         if (c > 1) {
             SRAI(s3, s1, c - 1);
@@ -1246,8 +1188,11 @@ void emit_shrd32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // Store sign for later use.
-        if (c == 1) SRLIxw(s4, s1, rex.w ? 63 : 31);
+        SRLIxw(s3, s1, rex.w ? 63 : 31);
+        XOR(s3, s3, s2);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
 
     SRLIxw(s3, s1, c);
@@ -1267,15 +1212,6 @@ void emit_shrd32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s5);
     }
-    IFX (X_OF) {
-        // the OF flag is set if a sign change occurred
-        if (c == 1) {
-            SRLI(s3, s1, rex.w ? 63 : 31);
-            XOR(s3, s3, s4);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     IFX (X_PF) {
         emit_pf(dyn, ninst, s1, s3, s4);
     }
@@ -1304,6 +1240,13 @@ void emit_shrd16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
         }
         return;
     }
+    IFX (X_OF) {
+        SRLI(s3, s1, 15);
+        XOR(s3, s3, s2);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
 
     // create concat first
     SLLI(s5, s2, 16);
@@ -1319,12 +1262,8 @@ void emit_shrd16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
         }
         OR(xFlags, xFlags, s3);
     }
-    IFX (X_OF) {
-        // Store sign for later use.
-        if (c == 1) SRLI(s4, s1, 15);
-    }
 
-    if (rv64_zbb) {
+    if (cpuext.zbb) {
         RORIW(s1, s1, c);
     } else {
         SRLI(s5, s1, c);
@@ -1339,16 +1278,6 @@ void emit_shrd16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s5);
     }
-    IFX (X_OF) {
-        // the OF flag is set if a sign change occurred
-        if (c == 1) {
-            SRLI(s3, s1, 15);
-            XOR(s3, s3, s4);
-            ANDI(s3, s3, 1);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     IFX (X_SF) {
         SLLIW(s3, s1, 16);
         SET_FLAGS_LTZ(s3, F_SF, s4, s5);
@@ -1361,48 +1290,36 @@ void emit_shrd16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
 
 void emit_shld32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uint32_t c, int s3, int s4, int s5)
 {
-    c &= (rex.w ? 0x3f : 0x1f);
     CLEAR_FLAGS();
     IFX (X_PEND) {
-        if (c) {
-            MOV64x(s3, c);
-            SDxw(s3, xEmu, offsetof(x64emu_t, op2));
-        } else
-            SDxw(xZR, xEmu, offsetof(x64emu_t, op2));
+        MOV64x(s3, c);
+        SDxw(s3, xEmu, offsetof(x64emu_t, op2));
         SDxw(s1, xEmu, offsetof(x64emu_t, op1));
         SET_DF(s4, rex.w ? d_shld64 : d_shld32);
     } else IFXORNAT (X_ALL) {
         SET_DFNONE();
     }
 
-    if (!c) {
-        if (!rex.w) {
-            ZEROUP(s1);
-        }
-        IFX (X_PEND) {
-            SDxw(s1, xEmu, offsetof(x64emu_t, res));
-        }
-        return;
-    }
     IFX (X_CF) {
-        if (c > 0) {
-            SRLI(s3, s1, (rex.w ? 64 : 32) - c);
-            ANDI(s4, s3, 1); // F_CF
-            OR(xFlags, xFlags, s4);
-        }
+        SRLI(s3, s1, (rex.w ? 64 : 32) - c);
+        ANDI(s4, s3, 1); // F_CF
+        OR(xFlags, xFlags, s4);
     }
+
     IFX (X_OF) {
-        // Store sign for later use.
-        if (c == 1) SRLIxw(s4, s1, rex.w ? 63 : 31);
+        SRLIxw(s3, s1, rex.w ? 62 : 30);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
 
     SLLIxw(s3, s1, c);
     SRLIxw(s1, s2, (rex.w ? 64 : 32) - c);
     OR(s1, s1, s3);
 
-    if (!rex.w) {
-        ZEROUP(s1);
-    }
+    if (!rex.w) ZEROUP(s1);
 
     IFX (X_PEND) {
         SDxw(s1, xEmu, offsetof(x64emu_t, res));
@@ -1410,15 +1327,6 @@ void emit_shld32c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s5);
     }
-    IFX (X_OF) {
-        // the OF flag is set if a sign change occurred
-        if (c == 1) {
-            SRLIxw(s3, s1, rex.w ? 63 : 31);
-            XOR(s3, s3, s4);
-            SLLI(s3, s3, F_OF2);
-            ORI(xFlags, xFlags, s3);
-        }
-    }
     IFX (X_SF) {
         SRLIxw(s3, s1, rex.w ? 63 : 31);
         SET_FLAGS_NEZ(s3, F_SF, s4);
@@ -1448,8 +1356,11 @@ void emit_shrd32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int
         OR(xFlags, xFlags, s3);
     }
     IFX (X_OF) {
-        // Store current sign for later use.
-        SRLIxw(s6, s1, rex.w ? 63 : 31);
+        SRLIxw(s3, s1, rex.w ? 63 : 31);
+        XOR(s3, s3, s2);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
     }
     ADDI(s4, xZR, (rex.w ? 64 : 32));
     SUB(s4, s4, s5);
@@ -1463,15 +1374,6 @@ void emit_shrd32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int
     if (!rex.w) {
         ZEROUP(s1);
     }
-    IFX (X_OF) {
-        ADDI(s5, s5, -1);
-        BNEZ_MARK(s5);
-        SRLIxw(s3, s1, rex.w ? 63 : 31);
-        XOR(s3, s3, s6);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-        MARK;
-    }
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s5);
     }
@@ -1505,8 +1407,12 @@ void emit_shld32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int
         OR(xFlags, xFlags, s4);
     }
     IFX (X_OF) {
-        // Store current sign for later use.
-        SRLIxw(s6, s1, rex.w ? 63 : 31);
+        SRLIxw(s6, s1, rex.w ? 62 : 30);
+        SRLI(s4, s6, 1);
+        XOR(s6, s6, s4);
+        ANDI(s6, s6, 1);
+        SLLI(s6, s6, F_OF2);
+        OR(xFlags, xFlags, s6);
     }
     SLLxw(s4, s1, s5);
     SRLxw(s3, s2, s3);
@@ -1518,16 +1424,6 @@ void emit_shld32(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, int
     if (!rex.w) {
         ZEROUP(s1);
     }
-
-    IFX (X_OF) {
-        ADDI(s5, s5, -1);
-        BNEZ_MARK(s5);
-        SRLIxw(s3, s1, rex.w ? 63 : 31);
-        XOR(s3, s3, s6);
-        SLLI(s3, s3, F_OF2);
-        OR(xFlags, xFlags, s3);
-        MARK;
-    }
     IFX (X_ZF) {
         SET_FLAGS_EQZ(s1, F_ZF, s5);
     }
@@ -1564,12 +1460,21 @@ void emit_shld16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
         return;
     }
 
+    IFX (X_OF) {
+        SRLI(s3, s1, 14);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     // create concat first
     SLLI(s5, s2, 16);
     OR(s1, s1, s5);
 
     IFX (X_CF) {
-        if (c < 16) {
+        if (c <= 16) {
             SRLI(s3, s1, 16 - c);
         } else {
             SRLI(s3, s2, 32 - c);
@@ -1577,12 +1482,7 @@ void emit_shld16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
         ANDI(s5, s3, 1); // LSB == F_CF
         OR(xFlags, xFlags, s5);
     }
-    IFX (X_OF) {
-        // Store sign for later use.
-        if (c == 1) SRLI(s5, s1, 15);
-    }
-
-    if (rv64_zbb) {
+    if (cpuext.zbb) {
         RORIW(s1, s1, 32 - c);
     } else {
         SLLI(s3, s1, c);
@@ -1594,16 +1494,6 @@ void emit_shld16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
     IFX (X_PEND) {
         SH(s1, xEmu, offsetof(x64emu_t, res));
     }
-    IFX (X_OF) {
-        // the OF flag is set if a sign change occurred
-        if (c == 1) {
-            SRLI(s3, s1, 15);
-            XOR(s3, s3, s5);
-            ANDI(s3, s3, 1);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     IFX (X_SF) {
         SLLIW(s4, s1, 16);
         SET_FLAGS_LTZ(s4, F_SF, s3, s5);
@@ -1617,6 +1507,119 @@ void emit_shld16c(dynarec_rv64_t* dyn, int ninst, rex_t rex, int s1, int s2, uin
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
 
+void emit_shld16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5, int s6)
+{
+    CLEAR_FLAGS();
+    IFX (X_PEND) {
+        SH(s1, xEmu, offsetof(x64emu_t, op1));
+        SH(s3, xEmu, offsetof(x64emu_t, op2));
+        SET_DF(s4, d_shld16);
+    } else IFXORNAT (X_ALL) {
+        SET_DFNONE();
+    }
+
+    IFX (X_OF) {
+        SRLI(s5, s1, 14);
+        SRLI(s4, s5, 1);
+        XOR(s5, s5, s4);
+        ANDI(s5, s5, 1);
+        SLLI(s5, s5, F_OF2);
+        OR(xFlags, xFlags, s5);
+    }
+
+    MOV32w(s6, 32);
+    SUBW(s6, s6, s3);
+    IFX (X_CF) {
+        SLLI(s5, s1, 16);
+        OR(s4, s2, s5);
+        SRLW(s5, s4, s6);
+        ANDI(s5, s5, 1); // LSB == F_CF
+        OR(xFlags, xFlags, s5);
+    }
+    // create concat first
+    SLLI(s5, s2, 16);
+    OR(s1, s1, s5);
+
+    if (cpuext.zbb) {
+        RORW(s1, s1, s6);
+    } else {
+        SLLW(s5, s1, s3);
+        SRLW(s1, s1, s6);
+        OR(s1, s1, s5);
+    }
+    ZEXTH(s1, s1);
+
+    IFX (X_PEND) {
+        SH(s1, xEmu, offsetof(x64emu_t, res));
+    }
+    IFX (X_SF) {
+        SLLIW(s4, s1, 16);
+        SET_FLAGS_LTZ(s4, F_SF, s4, s5);
+    }
+    IFX (X_ZF) {
+        SET_FLAGS_EQZ(s1, F_ZF, s4);
+    }
+    IFX (X_PF) {
+        emit_pf(dyn, ninst, s1, s4, s5);
+    }
+    if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s4, xZR);
+}
+
+void emit_shrd16(dynarec_rv64_t* dyn, int ninst, int s1, int s2, int s3, int s4, int s5, int s6)
+{
+    CLEAR_FLAGS();
+    IFX (X_PEND) {
+        SH(s1, xEmu, offsetof(x64emu_t, op1));
+        SH(s3, xEmu, offsetof(x64emu_t, op2));
+        SET_DF(s4, d_shrd16);
+    } else IFXORNAT (X_ALL) {
+        SET_DFNONE();
+    }
+
+    SLLI(s5, s2, 16);
+    OR(s1, s1, s5);
+    IFX (X_OF) {
+        SRLI(s5, s1, 15);
+        XOR(s5, s5, s2);
+        ANDI(s5, s5, 1);
+        SLLI(s5, s5, F_OF2);
+        OR(xFlags, xFlags, s5);
+    }
+
+    IFX (X_CF) {
+        ADDIW(s5, s3, -1);
+        SRLW(s5, s1, s5);
+        ANDI(s5, s5, 1); // LSB == F_CF
+        OR(xFlags, xFlags, s5);
+    }
+
+    if (cpuext.zbb) {
+        RORW(s1, s1, s3);
+    } else {
+        MOV32w(s6, 32);
+        SUBW(s6, s6, s3);
+        SRLW(s5, s1, s3);
+        SLLW(s1, s1, s6);
+        OR(s1, s1, s5);
+    }
+    ZEXTH(s1, s1);
+
+    IFX (X_PEND) {
+        SH(s1, xEmu, offsetof(x64emu_t, res));
+    }
+    IFX (X_ZF) {
+        SET_FLAGS_EQZ(s1, F_ZF, s5);
+    }
+    IFX (X_SF) {
+        SLLIW(s3, s1, 16);
+        SET_FLAGS_LTZ(s3, F_SF, s4, s5);
+    }
+    IFX (X_PF) {
+        emit_pf(dyn, ninst, s1, s4, s5);
+    }
+    if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
+}
+
 // emit RCL16 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
 void emit_rcl16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int s4)
 {
@@ -1624,6 +1627,16 @@ void emit_rcl16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
 
     SET_DFNONE();
 
+    IFX (X_OF) {
+        ANDI(xFlags, xFlags, ~(1UL << F_OF2));
+        SRLI(s3, s1, 14);
+        SRLI(s4, s3, 1);
+        XOR(s3, s3, s4);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
+
     c %= 17;
 
     ANDI(s3, xFlags, 1 << F_CF);
@@ -1636,36 +1649,32 @@ void emit_rcl16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
     SRLI(s1, s1, 17 - c);
     OR(s1, s1, s3);
 
-    IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
+    IFX (X_CF) {
+        ANDI(xFlags, xFlags, ~(1UL << F_CF));
         SRLI(s4, s4, 63);
-        IFX (X_CF) OR(xFlags, xFlags, s4);
+        OR(xFlags, xFlags, s4);
     }
 
-    IFX (X_OF) {
-        if (c == 1) {
-            SRLI(s3, s1, 15);
-            XOR(s3, s3, s4);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
 
 // emit RCR16 instruction, from s1 , constant c, store result in s1 using s3 and s4 as scratch
 void emit_rcr16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int s4)
 {
-    MAYUSE(s1);
-    MAYUSE(s3);
-    MAYUSE(s4);
-
     if (!(c % 17)) return;
 
     SET_DFNONE();
 
-    c %= 17;
+    IFX (X_OF) {
+        ANDI(xFlags, xFlags, ~(1UL << F_OF2));
+        SRLI(s3, s1, 15);
+        XOR(s3, s3, xFlags);
+        ANDI(s3, s3, 1);
+        SLLI(s3, s3, F_OF2);
+        OR(xFlags, xFlags, s3);
+    }
 
+    c %= 17;
 
     ANDI(s3, xFlags, 1 << F_CF);
     SLLI(s3, s3, 16);
@@ -1677,24 +1686,11 @@ void emit_rcr16c(dynarec_rv64_t* dyn, int ninst, int s1, uint32_t c, int s3, int
     SLLI(s4, s1, 47);
     ZEXTH(s1, s1);
 
-    IFX (X_CF | X_OF) {
-        ANDI(xFlags, xFlags, ~(1UL << F_CF | 1UL << F_OF2));
-    }
-
     IFX (X_CF) {
+        ANDI(xFlags, xFlags, ~(1UL << F_CF));
         SRLI(s4, s4, 63);
         OR(xFlags, xFlags, s4);
     }
 
-    IFX (X_OF) {
-        if (c == 1) {
-            SRLI(s3, s1, 14);
-            SRLI(s4, s3, 1);
-            XOR(s3, s3, s4);
-            ANDI(s3, s3, 1);
-            SLLI(s3, s3, F_OF2);
-            OR(xFlags, xFlags, s3);
-        }
-    }
     if (dyn->insts[ninst].nat_flags_fusion) NAT_FLAGS_OPS(s1, xZR, s3, xZR);
 }
