@@ -33,6 +33,7 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
     int v0, v1;
     int q0, q1;
     int d0, d1;
+    uint8_t tmp1, tmp2, tmp3;
     int64_t fixedaddress, gdoffset;
     int unscaled;
     int64_t j64;
@@ -174,6 +175,34 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
             }
             break;
 
+#define GO(GETFLAGS, NO, YES, NATNO, NATYES, F)                                                  \
+    READFLAGS_FUSION(F, x1, x2, x3, x4, x5);                                                     \
+    if (!dyn->insts[ninst].nat_flags_fusion) {                                                   \
+        GETFLAGS;                                                                                \
+    }                                                                                            \
+    nextop = F8;                                                                                 \
+    GETGD;                                                                                       \
+    if (MODREG) {                                                                                \
+        ed = TO_NAT((nextop & 7) + (rex.b << 3));                                                \
+        if (dyn->insts[ninst].nat_flags_fusion) {                                                \
+            NATIVEMV(NATYES, gd, ed);                                                            \
+        } else {                                                                                 \
+            MV##YES(gd, ed, tmp1);                                                               \
+        }                                                                                        \
+        if (!rex.w) ZEROUP(gd);                                                                  \
+    } else {                                                                                     \
+        addr = geted(dyn, addr, ninst, nextop, &ed, tmp2, tmp3, &fixedaddress, rex, NULL, 1, 0); \
+        if (dyn->insts[ninst].nat_flags_fusion) {                                                \
+            NATIVEJUMP(NATNO, 8);                                                                \
+        } else {                                                                                 \
+            B##NO(tmp1, 8);                                                                      \
+        }                                                                                        \
+        LDxw(gd, ed, fixedaddress);                                                              \
+    }
+
+            GOCOND(0x40, "CMOV", "Gd, Ed");
+#undef GO
+
         case 0x51:
             INST_NAME("SQRTSS Gx, Ex");
             nextop = F8;
@@ -303,7 +332,7 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
             FEQS(x3, d1, d1);
             AND(x2, x2, x3);
             BEQ_MARK(x2, xZR);
-            FLTS(x2, d1, d0);
+            FLES(x2, d1, d0);
             BEQ_MARK2(x2, xZR);
             MARK;
             FMVS(d0, d1);
@@ -336,7 +365,7 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
             FEQS(x3, d1, d1);
             AND(x2, x2, x3);
             BEQ_MARK(x2, xZR);
-            FLTS(x2, d0, d1);
+            FLES(x2, d0, d1);
             BEQ_MARK2(x2, xZR);
             MARK;
             FMVS(d0, d1);
@@ -425,15 +454,32 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
         case 0xAE:
             nextop = F8;
             switch ((nextop >> 3) & 7) {
+                case 0:
+                case 1:
+                    if(rex.is32bits || !MODREG) {
+                        INST_NAME("Illegal AE");
+                        FAKEED;
+                        UDF();
+                    } else {
+                        if(((nextop>>3)&7)==1) {INST_NAME("RDGSBASE");} else {INST_NAME("RDFSBASE");}
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                        int seg = _FS + ((nextop>>3)&7);
+                        grab_segdata(dyn, addr, ninst, x4, seg, (MODREG));
+                        MV(ed, x4);
+                    }
+                     break;
                 case 2:
-                    INST_NAME("(unsupported) WRFSBASE Ed");
-                    FAKEED;
-                    UDF();
-                    break;
                 case 3:
-                    INST_NAME("(unsupported) WRGSBASE Ed");
-                    FAKEED;
-                    UDF();
+                    if(rex.is32bits || !MODREG) {
+                        INST_NAME("Illegal AE");
+                        FAKEED;
+                        UDF();
+                    } else {
+                        if(((nextop>>3)&7)==3) {INST_NAME("WRGSBASE");} else {INST_NAME("WRFSBASE");}
+                        ed = TO_NAT((nextop & 7) + (rex.b << 3));
+                        int seg = _FS + ((nextop>>3)&7)-2;
+                        SD(ed, xEmu, offsetof(x64emu_t, segs_offs[seg]));
+                    }
                     break;
                 case 5:
                     INST_NAME("(unsupported) INCSSPD/INCSSPQ Ed");
@@ -466,7 +512,7 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
             MOV32w(gd, 0);
             B_NEXT_nocond;
             MARK;
-            if (rv64_zbb) {
+            if (cpuext.zbb) {
                 CPOPxw(gd, ed);
             } else {
                 if (rex.w) {
@@ -554,46 +600,53 @@ uintptr_t dynarec64_F30F(dynarec_rv64_t* dyn, uintptr_t addr, uintptr_t ip, int
             break;
         case 0xBC:
             INST_NAME("TZCNT Gd, Ed");
-            SETFLAGS(X_ZF, SF_SUBSET, NAT_FLAGS_NOFUSION);
+            if (!BOX64DRENV(dynarec_safeflags)) {
+                SETFLAGS(X_CF | X_ZF, SF_SUBSET, NAT_FLAGS_NOFUSION);
+            } else {
+                SETFLAGS(X_ALL, SF_SET, NAT_FLAGS_NOFUSION);
+            }
             SET_DFNONE();
             nextop = F8;
             GETED(0);
             GETGD;
+            CLEAR_FLAGS();
             if (!rex.w && MODREG) {
                 ZEXTW2(x4, ed);
                 ed = x4;
             }
-            ANDI(xFlags, xFlags, ~((1 << F_ZF) | (1 << F_CF)));
-            BNE_MARK(ed, xZR);
-            ORI(xFlags, xFlags, 1 << F_CF);
+            IFX (X_CF) SET_FLAGS_EQZ(ed, F_CF, x3);
+            BNEZ_MARK(ed);
+            IFX (X_ZF) ANDI(xFlags, xFlags, ~(1 << F_ZF));
             MOV32w(gd, rex.w ? 64 : 32);
             B_NEXT_nocond;
             MARK;
             CTZxw(gd, ed, rex.w, x3, x5);
-            BNE(gd, xZR, 4 + 4);
-            ORI(xFlags, xFlags, 1 << F_ZF);
+            IFX (X_ZF) SET_FLAGS_EQZ(gd, F_ZF, x3);
             break;
         case 0xBD:
             INST_NAME("LZCNT Gd, Ed");
-            SETFLAGS(X_ZF | X_CF, SF_SUBSET, NAT_FLAGS_NOFUSION);
+            if (!BOX64DRENV(dynarec_safeflags)) {
+                SETFLAGS(X_CF | X_ZF, SF_SUBSET, NAT_FLAGS_NOFUSION);
+            } else {
+                SETFLAGS(X_ALL, SF_SET, NAT_FLAGS_NOFUSION);
+            }
             SET_DFNONE();
             nextop = F8;
             GETED(0);
             GETGD;
+            CLEAR_FLAGS();
             if (!rex.w && MODREG) {
                 ZEXTW2(x4, ed);
                 ed = x4;
             }
-            BNE_MARK(ed, xZR);
+            IFX (X_CF) SET_FLAGS_EQZ(ed, F_CF, x3);
+            BNEZ_MARK(ed);
             MOV32w(gd, rex.w ? 64 : 32);
-            ANDI(xFlags, xFlags, ~(1 << F_ZF));
-            ORI(xFlags, xFlags, 1 << F_CF);
+            IFX (X_ZF) ANDI(xFlags, xFlags, ~(1 << F_ZF));
             B_NEXT_nocond;
             MARK;
             CLZxw(gd, ed, rex.w, x5, x2, x3);
-            ANDI(xFlags, xFlags, ~((1 << F_ZF) | (1 << F_CF)));
-            BNE(gd, xZR, 4 + 4);
-            ORI(xFlags, xFlags, 1 << F_ZF);
+            IFX (X_ZF) SET_FLAGS_EQZ(gd, F_ZF, x3);
             break;
 
         case 0xC2:
