@@ -5,7 +5,7 @@ typedef struct x64emu_s x64emu_t;
 typedef struct dynablock_s dynablock_t;
 
 uint32_t X31_hash_code(void* addr, int len);
-void FreeDynablock(dynablock_t* db, int need_lock);
+void FreeDynablock(dynablock_t* db, int need_lock, int need_remove);
 void MarkDynablock(dynablock_t* db);
 void MarkRangeDynablock(dynablock_t* db, uintptr_t addr, uintptr_t size);
 int FreeRangeDynablock(dynablock_t* db, uintptr_t addr, uintptr_t size);
@@ -20,11 +20,13 @@ dynablock_t* DBAlternateBlock(x64emu_t* emu, uintptr_t addr, uintptr_t filladdr,
 
 // for use in signal handler
 void cancelFillBlock(void);
+void dynablock_leave_runtime(dynablock_t* db);
 
 // clear instruction cache on a range
 void ClearCache(void* start, size_t len);
 
 uintptr_t getX64Address(dynablock_t* db, uintptr_t native_addr);
 int getX64AddressInst(dynablock_t* db, uintptr_t x64pc);
+uintptr_t getX64InstAddress(dynablock_t* db, int inst);
 
 #endif //__DYNABLOCK_H_
\ No newline at end of file
