@@ -8,12 +8,16 @@ NTSYSAPI ULONG WINAPI NtGetTickCount(VOID);
 NTSYSAPI ULONG NTAPI RtlRandom(ULONG *seed);
 
 static int nCPU = 0;
+static int read_ncpu = 0;
 
 void grabNCpu() {
     SYSTEM_INFO sysinfo;
     GetSystemInfo(&sysinfo);
     nCPU = sysinfo.dwNumberOfProcessors;
 }
+int get_cpuMhz() {
+    return 2000;    //TODO: find a way to get realy value instead of arbitrary default to 2.0GHz
+}
 int getNCpu()
 {
     if(!nCPU)
@@ -22,6 +26,16 @@ int getNCpu()
         return BOX64ENV(maxcpu);
     return nCPU;
 }
+int getNCpuUnmasked()
+{
+    if(!nCPU)
+        grabNCpu();
+    return nCPU;
+}
+int canNCpuBeChanged()
+{
+    return read_ncpu?0:1;
+}
 
 const char* getBoxCpuName()
 {
