@@ -47,11 +47,11 @@ GO(g_async_queue_length, iFp)
 GO(g_async_queue_length_unlocked, iFp)
 GO(g_async_queue_lock, vFp)
 GO(g_async_queue_new, pFv)
-//GOM(g_async_queue_new_full, pFp)
+GOM(g_async_queue_new_full, pFEp)
 GO(g_async_queue_pop, pFp)
 GO(g_async_queue_pop_unlocked, pFp)
 GO(g_async_queue_push, vFpp)
-//GOM(g_async_queue_push_sorted, vFpppp)
+GOM(g_async_queue_push_sorted, vFEpppp)
 //GOM(g_async_queue_push_sorted_unlocked, vFpppp)
 GO(g_async_queue_push_unlocked, vFpp)
 GO(g_async_queue_ref, pFp)
@@ -614,6 +614,8 @@ GO(g_log_set_always_fatal, iFi)
 GOM(g_log_set_default_handler, pFEpp)
 GO(g_log_set_fatal_mask, iFpi)
 GOM(g_log_set_handler, uFEpipp)
+GOM(g_log_set_writer_func, vFEppp)
+GO(g_log_writer_default, iFipLp)
 GO(g_log_writer_is_journald, iFi)
 GOM(g_logv, vFEpipA)
 GO(g_log_structured_array, vFipL)
@@ -628,7 +630,7 @@ GO(g_main_context_find_source_by_id, pFpu)  // sould wrap?
 GO(g_main_context_find_source_by_user_data, pFpp)   // sould wrap?
 GOM(g_main_context_get_poll_func, pFEp)
 GO(g_main_context_get_thread_default, pFv)
-//GOM(g_main_context_invoke, vFppp)  // 2nd is GSourceFuncs
+GOM(g_main_context_invoke, vFppp)  // 2nd is GSourceFuncs
 //GOM(g_main_context_invoke_full, vFpippp)  // 3rd is GSourceFuncs
 GO(g_main_context_is_owner, iFp)
 GO(g_main_context_iteration, iFpi)
@@ -853,7 +855,7 @@ GO(g_queue_find, pFpp)
 GOM(g_queue_find_custom, pFEppp)
 GOM(g_queue_foreach, vFEppp)
 GO(g_queue_free, vFp)
-//GOM(g_queue_free_full, vFEpp)
+GOM(g_queue_free_full, vFEpp)
 GO(g_queue_get_length, uFp)
 GO(g_queue_index, iFpp)
 GO(g_queue_init, vFp)
@@ -1176,6 +1178,7 @@ GO(g_string_down, pFp)
 GO(g_string_equal, iFpp)
 GO(g_string_erase, pFpll)
 GO(g_string_free, pFpi)
+GO(g_string_free_and_steal, pFp)
 GO(g_string_free_to_bytes, pFp)
 GO(g_string_hash, uFp)
 GO(g_string_insert, pFplp)
@@ -1278,20 +1281,20 @@ GO(g_thread_get_initialized, iFv)
 //GO(g_thread_init_glib, 
 GO(g_thread_join, pFp)
 GOM(g_thread_new, pFEppp)
-//GOM(g_thread_pool_free, vFpii)
+GO(g_thread_pool_free, vFpii)
 GO(g_thread_pool_get_max_idle_time, uFv)
-//GOM(g_thread_pool_get_max_threads, iFp)
+GO(g_thread_pool_get_max_threads, iFp)
 GO(g_thread_pool_get_max_unused_threads, iFv)
-//GOM(g_thread_pool_get_num_threads, uFp)
+GO(g_thread_pool_get_num_threads, uFp)
 GO(g_thread_pool_get_num_unused_threads, uFv)
-//GOM(g_thread_pool_new, pFppiip)
-//GOM(g_thread_pool_push, iFppp)
+GOM(g_thread_pool_new, pFEppiip)
+GO(g_thread_pool_push, iFppp)
 GO(g_thread_pool_set_max_idle_time, vFu)
-//GOM(g_thread_pool_set_max_threads, iFpip)
+GO(g_thread_pool_set_max_threads, iFpip)
 GO(g_thread_pool_set_max_unused_threads, vFi)
-//GOM(g_thread_pool_set_sort_function, vFppp)
+GOM(g_thread_pool_set_sort_function, vFEppp)
 GO(g_thread_pool_stop_unused_threads, vFv)
-//GOM(g_thread_pool_unprocessed, uFp)
+GO(g_thread_pool_unprocessed, uFp)
 GO(g_thread_ref, pFp)
 GO(g_thread_self, pFv)
 GO(g_thread_set_priority, vFpu)
@@ -1421,6 +1424,7 @@ GO(g_utf8_find_next_char, pFpp)
 GO(g_utf8_find_prev_char, pFpp)
 GO(g_utf8_get_char, uFp)
 GO(g_utf8_get_char_validated, uFpl)
+GO(g_utf8_make_valid, pFpi)
 GO(g_utf8_normalize, pFplu)
 GO(g_utf8_offset_to_pointer, pFpl)
 GO(g_utf8_pointer_to_offset, lFpp)
